﻿#!/usr/bin/python3
# -*- coding: utf-8 -*-
import tkinter as tk
import tkinter.ttk as ttk
from tkinter import filedialog
import sys
import os
import codecs
from pathlib import Path
from PIL import Image, ImageTk

import thumbnail

class PicList:
    def __init__(self, base):
        self.base = base
        self.item_num = 0
        self.max_id = 0
        self.pitem = {}
        self.default_effect = 0
        self.default_caption = ''
        self.content_dir = ''
        
        ttk.Style().configure('Treeview', rowheight=100)
        self.tree = ttk.Treeview(self.base, column=('A', 'B'), height=14)

        ysb = ttk.Scrollbar(self.tree, orient=tk.VERTICAL, command=self.tree.yview)
        ysb.place(relheight=0.95,relwidth=0.05,relx=0.95,rely=0.0)
        self.tree.configure(yscroll=ysb.set)
        
        self.tree.heading('#0', text="Thumbnail")
        self.tree.heading('#1', text="Effect")
        self.tree.heading('#2', text="Caption")

        self.tree.column('#0', width=150)
        self.tree.column('#1', width=30)
        self.tree.column('#2', width=200)

        self.tree.bind('<ButtonPress-1>', self.buttonDown)
        self.tree.bind('<ButtonRelease-1>', self.buttonUp, add='+')
        self.tree.bind('<B1-Motion>', self.buttonMove, add='+')

    def buttonDown(self, event):
        self.tv = event.widget
        if self.tv.identify_row(event.y) not in self.tv.selection():
            self.tv.selection_set(self.tv.identify_row(event.y))

    def buttonUp(self, event):
        self.tv = event.widget
        if self.tv.identify_row(event.y) in self.tv.selection():
            self.tv.selection_set(self.tv.identify_row(event.y))

    def buttonMove(self, event):
        self.tv = event.widget
        self.moveto = self.tv.index(self.tv.identify_row(event.y))
        for s in self.tv.selection():
            self.tv.move(s, '', self.moveto)

    def addMenu(self, menu):
        self.rclick = menu
        self.tree.bind('<Button-3>', self.rclick.popup)

    def addDialog(self, dialog):
        self.dialog = dialog

    def insert(self, filename, num, text):
        self.pitem[str(self.max_id)] = PicListItem(filename, num, text, str(self.max_id))
        self.tree.insert('', 'end', iid=str(self.max_id), image=self.pitem[str(self.max_id)].image, value = (num, text), tags=str(self.max_id))

        self.item_num = self.item_num + 1
        if self.item_num % 2 == 1:
            self.tree.tag_configure(self.max_id, background='#e0e0e0')
            
        self.max_id = self.max_id + 1

    def update(self, tree_item, num, text):
        self.tree_item = tree_item
        self.currInd = str(self.tree.item(tree_item)['tags'][0])

        if self.tree_item:
            self.tree.item(tree_item, value=(num, text))
            self.pitem[self.currInd].update(num, text, self.currInd)

    def insert_folder(self, path, thumb_path):
        self.content_dir = path
        p = Path(path)
        for f in p.glob('*.jpg'):
            self.insert(os.path.join(p, f.name), self.default_effect, self.default_caption)

    def pack(self, **kwargs):
        self.tree.pack(kwargs)

class PicListItem:
    def __init__(self, pic_file, num, text, tag):
        if not os.path.exists(pic_file):
            print('Error: File not found: {}'.format(pic_file), file=sys.stderr)

        self.basename = os.path.basename(pic_file)
        self.dirname = os.path.dirname(pic_file)
        self.thumbnail = os.path.join(self.dirname, 'thumbnail', self.basename)
        
        if not os.path.exists(self.thumbnail):
            print('Error: Thumbnail not found: {}'.format(self.thumbnail), file=sys.stderr)

        self.image = ImageTk.PhotoImage(Image.open(self.thumbnail))

        self.mp4name = os.path.splitext(os.path.basename(pic_file))[0]
        self.mp4path = os.path.join(self.dirname, self.mp4name)
        if os.path.exists(self.mp4path):
            self.pic_file = self.mp4name
        else:
            self.pic_file = self.basename

        self.num = num
        self.text = text
        self.tag = tag

    def update(self, num, text, tag):
        self.num = num
        self.text = text
        self.tag = tag

class RightClickMenu:
    def __init__(self, master, app):
        self.master = master
        self.app = app
        self.aMenu = tk.Menu(master, tearoff=0)
        self.aMenu.add_command(label='Edit', command=self.edit)
        self.aMenu.add_command(label='Delete', command=self.delete)

        self.tree_item = ''

    def delete(self):
        if self.tree_item:
            self.app.tree.delete(self.tree_item)

    def edit(self):
        self.app.dialog.open(self.tree_item)

    def popup(self, event):
        self.aMenu.post(event.x_root, event.y_root)
        self.tree_item = self.app.tree.focus()

class EditDialog:
    def __init__(self, master, app):
        self.master = master
        self.app = app

    def open(self, tree_item):
        self.tree_item = tree_item
        self.sub_win = tk.Toplevel(master=self.master)

        self.currInd = str(self.app.tree.item(tree_item)['tags'][0])
        self.num = self.app.pitem[self.currInd].num
        self.text = self.app.pitem[self.currInd].text

        frame1 = tk.Frame(self.sub_win)
        frame1.pack(padx=5, pady=5)
        self.label1 = tk.Label(frame1, text='Effect:')
        self.label1.pack(side='left', anchor=tk.W)
        self.entry1 = tk.Entry(frame1, bd=1)
        self.entry1.pack(side='left', anchor=tk.W)
        self.entry1.insert('end', self.num)

        frame2 = tk.Frame(self.sub_win)
        frame2.pack(padx=5, pady=5)
        self.label2 = tk.Label(frame2, text='Caption:')
        self.label2.pack(side='left', anchor=tk.W)
        self.entry2 = tk.Entry(frame2, bd=1, width=40)
        self.entry2.pack(side='left', anchor=tk.W)
        self.entry2.insert('end', self.text)

        frame3 = tk.Frame(self.sub_win)
        frame3.pack(padx=5, pady=5)
        self.button_ok = tk.Button(frame3, text='OK', command=self.ok)
        self.button_ok.pack(side='left')
        self.button_cancel = tk.Button(frame3, text='Cancel', command=self.cancel)
        self.button_cancel.pack(side='left')

        self.sub_win.bind('<Return>', self.key_ret)

        self.entry2.focus_set()

    def key_ret(self, event):
        self.ok()

    def ok(self):
        self.txt_effect = self.entry1.get()
        self.txt_caption = self.entry2.get()
        self.app.update(self.tree_item, self.txt_effect, self.txt_caption)
        self.sub_win.destroy()
        
    def cancel(self):
        self.sub_win.destroy()

class AppMenus:
    def __init__(self, base, app):
        self.menu_bar = tk.Menu(base)
        self.base = base
        self.app = app
        
        self.base.config(menu=self.menu_bar)

        self.file_menu = tk.Menu(self.menu_bar, tearoff=0)
        self.menu_bar.add_cascade(label='File', menu=self.file_menu)
        self.file_menu.add_command(label='New', command=self.file_new)
        self.file_menu.add_command(label='Open Folder', command=self.file_open_folder)
        self.file_menu.add_command(label='Open Script', command=self.file_open_script)
        self.file_menu.add_command(label='Save Script', command=self.file_save)
        self.file_menu.add_separator()
        self.file_menu.add_command(label='Exit', command=self.file_exit)

    def file_new(self):
        pass

    def file_open_folder(self):
        self.dir = tk.filedialog.askdirectory()
        if not os.path.exists(self.dir):
            print('Error: No image folder exists.', file=sys.stderr)
            sys.exit(1)

        thumbnail.make_thumbnail(self.dir)

        # Picture list
        self.app.pack(side=tk.TOP, fill=tk.BOTH, expand=1)

        self.rcm = RightClickMenu(self.base, self.app)
        self.app.addMenu(self.rcm)

        self.ed = EditDialog(self.base, self.app)
        self.app.addDialog(self.ed)

        self.app.insert_folder(self.dir, self.dir)

    def file_open_script(self):
        pass

    def file_save(self):
        #self.base.withdraw()
        
        self.file_type = [("","*.txt")]
        self.iniDir = os.path.abspath(os.path.dirname(__file__))
        self.file = tk.filedialog.asksaveasfilename(filetypes=self.file_type, initialdir=self.iniDir)

        self.fout = codecs.open(self.file, 'w', 'shift_jis')
        self.fout.write('DIR,{}\r\n'.format(self.app.content_dir))
        for ch in self.app.tree.get_children():
            self.fout.write('image,{},{},{}\r\n'.format(self.app.pitem[str(ch)].pic_file, self.app.pitem[str(ch)].num, self.app.pitem[str(ch)].text))
        self.fout.close()

    def file_exit(self):
        sys.exit(0)

def main():
    base = tk.Tk()
    base.title('Movie Maker')
    base.geometry('500x700')

    pl = PicList(base)
    menus = AppMenus(base, pl)

    base.mainloop()

if __name__ == "__main__":
    main()
